﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;

namespace Clocks
{
    public partial class Form1 : Form
    {
        private ArrayList clocks;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            clocks = new ArrayList();
            RenderClocks();
        }

        private void RenderClocks()
        {
            foreach (ToolStripMenuItem t in contextMenuStrip1.Items)
            {
                if (t.Checked)
                {
                    Label l;
                    TimeZoneInfo tzi;

                    switch (t.Text)
                    {
                        case "Bragg":
                            l = Clock_Bragg;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Eastern Standard Time");
                            break;
                        case "Iraq":
                            l = Clock_Iraq;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Arabic Standard Time");
                            break;
                        case "Afghanistan":
                            l = Clock_Afghanistan;
                            tzi = TimeZoneInfo.CreateCustomTimeZone("Afghanistan",
                                new TimeSpan(04, 30, 00), "(GTM+04:30) Afghanistan", "Afghanistan");
                            break;
                        case "Hawaii":
                            l = Clock_Hawaii;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Hawaiian Standard Time");
                            break;
                        case "Korea":
                            l = Clock_Korea;
                            tzi = TimeZoneInfo.FindSystemTimeZoneById("Korea Standard Time");
                            break;
                        default:
                            l = null;
                            tzi = null;
                            break;
                    }

                    if (l != null && tzi != null)
                    {
                        myClock c = new myClock(l, tzi, t.Text);
                        Thread tc = new Thread(c.runClock);
                        c.tClock.Start();
                        clocks.Add(c);
                    }
                }
            }
        }

        private void Form1_FormClosing(Object sender, FormClosingEventArgs e)
        {
            foreach (myClock c in clocks.ToArray())
            {
                c.closeThread();
            }

            Application.Exit();
        }

        private void braggToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Bragg.Visible = !Clock_Bragg.Visible;
        }

        private void iraqToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Iraq.Visible = !Clock_Iraq.Visible;
        }

        private void afghanistanToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Afghanistan.Visible = !Clock_Afghanistan.Visible;
        }

        private void hawaiiToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Hawaii.Visible = !Clock_Hawaii.Visible;
        }

        private void koreaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem t = (ToolStripMenuItem)sender;
            t.Checked = !t.Checked;
            Clock_Korea.Visible = !Clock_Korea.Visible;
        }

        private void Form1_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                contextMenuStrip1.Show(MousePosition);
            }
        }
    }

    public class myClock
    {
        private Label l;
        private TimeZoneInfo tzi;
        private bool closing;
        private string location;

        public Thread tClock;

        public myClock(Label myLabel, TimeZoneInfo tzi, string location)
        {
            this.l = myLabel;
            this.tzi = tzi;
            this.tClock = new Thread(this.runClock);
            this.closing = false;
            this.location = location;
        }

        private void SetText(string text)
        {
            
            this.l.Text = text;
        }

        public void closeThread()
        {
            this.closing = true;
        }

        public delegate void SetTextCallback(string text);

        public void runClock()
        {
            DateTime thisTime;

            while (!closing)
            {
                thisTime = DateTime.Now;
                thisTime = TimeZoneInfo.ConvertTime(thisTime, TimeZoneInfo.Local, tzi);

                if (l.InvokeRequired)
                {
                    this.l.Invoke (new SetTextCallback(this.SetText),
                        new object[] { string.Format("{0} {1:d2}:{2:d2}",
                            this.location, thisTime.Hour, thisTime.Minute) });
                }
                else
                {
                    SetText(thisTime.Hour + ":" + thisTime.Minute);
                }
                Thread.Sleep(1000);
            }
        }
    }
}
